const walkerClass = require('./walker.js'),
OPT = require ('./options.js');

let includedDirs=[],
excludedDirs=[],
totalProcessedFiles=0,
totalProcessedDirs=0,
debug = false,

walker = new walkerClass();
walker.on('start',    cbStart);
walker.on('error',    cbError);
walker.on('file',     cbFile);
walker.on('dir',      cbDir);
walker.on('unknown',  cbUnknow);
walker.on('pause',    cbPause);
walker.on('resume',   cbResume);
walker.on('duplicate',cbDuplicate);
walker.on('done',     cbDone);

process.on('message',(m)=>{
 switch (m.walker){
  case OPT.START:
   debug = m.debug ? true : false;
   start(m.dirs);
  break;

  case OPT.PAUSE:
   walker.pause();
  break;
  
  case OPT.RESUME:
   walker.resume();
  break;
  
  case OPT.RESET:
   reset();
  break;
 }
})
function start(dirs){
 debug&&console.log('walkerHelper.start: '+dirs);
 reset();
 totalProcessedFiles=0,
 totalProcessedDirs=0;

 for (let i in dirs){
  let dir = dirs[i];
  if (dir.isIncluded)
   includedDirs.push(dir.path);
  else 
   excludedDirs.push(dir.path);
 }

 walker.addToQueue(includedDirs);
 walker.filterDir((dir,stat)=>{
  return excludedDirs.indexOf(dir) !== -1
 });
 
 walker.start();
}

function reset(){
 debug&&console.log('walkerHelper.reset');
 walker.reset();
 includedDirs=[],
 excludedDirs=[];
}

function cbStart(){
  debug&&console.log('walkerHelper.cbStart');
 let obj = {
  walker:OPT.STARTED
 }

 process.send(obj);
}
function cbError(err,entry,stat){
 debug&&console.log('walkerHelper.cbError: '+err+ ', entry: '+entry);
 //Log errors
}
function cbUnknow(entry,stat){
 debug&&console.log('walkerHelper.cbUnknow: entry: '+entry);
 //Log errors 
}

function cbFile(entry,stat){
 totalProcessedFiles++;
}

function cbDir(entry,stat){
 totalProcessedDirs++
 
 let obj = {
  walker:   OPT.DIR,
  dir:      entry,
  stat:     stat,
  totalDirs:totalProcessedDirs
 }
 process.send(obj);
}

function cbPause(){
 debug&&console.log('walkerHelper.cbPause');
 let obj = {
  walker:OPT.PAUSED
 }
 process.send(obj);
}

function cbResume(){
 debug&&console.log('walkerHelper.cbResume');
 let obj = {
  walker: OPT.RESUMED
 }
 process.send(obj);
}

function cbDuplicate(files,size,ext,hash){
 debug&&console.log('walkerHelper.cbDuplicate');
 let df = JSON.stringify({size:size, id:hash+ext+size, files:files});
 
 let obj = {
  walker:    OPT.WEBVIEW,
  addRows:   'addRows('+df+')'
 }
 process.send(obj);
}

function cbDone(){
 debug&&console.log('walkerHelper.cbDone');
 let obj = {
  walker:    OPT.DONE,
  totalFiles:totalProcessedFiles,
  totalDirs: totalProcessedDirs
 }
  process.send(obj);
  this.reset();
}
